

from raspi_system import hwinfo
from twisted.internet.defer import inlineCallbacks
from .base import SysInfoBase


class HostInfo(SysInfoBase):
    def __init__(self, *args):
        super(HostInfo, self).__init__(*args)
        self._hostname = None
        self._machine_id = None
        self._boot_id = None
        self._operating_system = None
        self._kernel = None
        self._architecture = None
        self._hardware_vendor = None
        self._hardware_model = None

    def install(self):
        super(HostInfo, self).install()
        self._items = {
            'hostname': 'hostname',
            'machine_id': 'machine_id',
            'boot_id': 'boot_id',
            'operating_system': 'operating_system',
            'kernel': 'kernel',
            'architecture': 'architecture',
            'hardware_vendor': 'hardware_vendor',
            'hardware_model': 'hardware_model'
        }

    _hnc_items = {
        'static hostname': '_hostname',
        'machine id': '_machine_id',
        'boot id': '_boot_id',
        'operating system': '_operating_system',
        'kernel': '_kernel',
        'architecture': '_architecture',
        'hardware vendor': '_hardware_vendor',
        'hardware model': '_hardware_model',
    }

    def _hostnamectl(self):
        def _parse_result(result):
            rval = {}
            for row in result.split(b'\n'):
                try:
                    tag, value = row.split(b':')
                except ValueError:
                    continue
                tag = tag.strip().lower().decode('utf-8')
                value = value.strip().decode('utf-8')
                if tag in self._hnc_items.keys():
                    rval[self._hnc_items[tag]] = value
            return rval
        d = self._shell_execute(['hostnamectl'], _parse_result)

        def _save_result(result):
            for k, v in result.items():
                setattr(self, k, v)

        d.addCallback(_save_result)
        return d

    @inlineCallbacks
    def hostname(self):
        if not self._hostname:
            yield self._hostnamectl()
        return self._hostname

    @inlineCallbacks
    def machine_id(self):
        if not self._machine_id:
            yield self._hostnamectl()
        return self._machine_id

    @inlineCallbacks
    def boot_id(self):
        if not self._boot_id:
            yield self._hostnamectl()
        return self._boot_id

    @inlineCallbacks
    def operating_system(self):
        if not self._operating_system:
            yield self._hostnamectl()
        return self._operating_system

    @inlineCallbacks
    def kernel(self):
        if not self._kernel:
            yield self._hostnamectl()
        return self._kernel

    @inlineCallbacks
    def architecture(self):
        if not self._architecture:
            yield self._hostnamectl()
        return self._architecture

    @inlineCallbacks
    def hardware_vendor(self):
        if not self._hardware_vendor:
            yield self._hostnamectl()
        if not self._hardware_vendor and self.actual.config.platform == 'rpi':
            if hwinfo.is_pi3() or hwinfo.is_pi4():
                self._hardware_vendor = "Raspberry Pi Foundation"
        return self._hardware_vendor

    @inlineCallbacks
    def hardware_model(self):
        if not self._hardware_model:
            yield self._hostnamectl()
        if not self._hardware_model and self.actual.config.platform == 'rpi':
            if hwinfo.is_pi3() or hwinfo.is_pi4():
                self._hardware_model = "{} {}".format(hwinfo.model_string(), hwinfo.model_revcode())
        return self._hardware_model





import re
import psutil
from functools import partial
from .base import SysInfoBase


class TemperatureInfo(SysInfoBase):
    def __init__(self, *args):
        super(TemperatureInfo, self).__init__(*args)

    def install(self):
        super(TemperatureInfo, self).install()
        result = psutil.sensors_temperatures()
        for device in result.keys():
            zones = result[device]
            if len(zones) == 1:
                self._items[device] = partial(self._read_temp, device, 0)
            else:
                self._items[device] = SysInfoBase(self.actual)
                for idx, zone in enumerate(zones):
                    self._items[device].items[zone.label] = partial(self._read_temp, device, idx)
        if self.actual.config.platform == 'rpi':
            self._items['gpu'] = self._read_raspi_gpu_temp

    def _read_temp(self, device, zone):
        return psutil.sensors_temperatures()[device][zone].current

    def _read_raspi_gpu_temp(self):
        def _handle_result(result):
            return float(re.findall(r"[\d.]+", result.decode())[0])
        d = self._shell_execute(['vcgencmd', 'measure_temp'], _handle_result)
        return d





from .base import SysInfoBase
from .versions import VersionsInfo
from .config import ConfigInfo


class AppInfo(SysInfoBase):
    def __init__(self, *args):
        super(AppInfo, self).__init__(*args)
        self._app_name = None
        self._app_class = None
        self._versions = None

    def install(self):
        super(AppInfo, self).install()
        self._items = {'name': 'app_name',
                       'class': 'app_class',
                       'versions': VersionsInfo(self.actual),
                       'config': ConfigInfo(self.actual)}

    def app_name(self):
        if not self._app_name:
            self._app_name = self.actual.config.appname
        return self._app_name

    def app_class(self):
        if not self._app_class:
            self._app_class = str(self.actual.__class__)
        return self._app_class





from twisted.internet.defer import inlineCallbacks


class SysInfoBase(object):
    def __init__(self, actual):
        self._log = None
        self._actual = actual
        self._items = {}

    @property
    def log(self):
        return self._log or self._actual.log

    @property
    def items(self):
        return self._items

    @property
    def actual(self):
        if hasattr(self._actual, '_actual'):
            return self._actual._actual
        else:
            return self._actual

    def _shell_execute(self, *args, **kwargs):
        return self._actual._shell_execute(*args, **kwargs)

    def install(self):
        pass

    @inlineCallbacks
    def render(self):
        rval = {}
        for k, v in self.items.items():
            if callable(v):
                rval[k] = yield v()
            elif hasattr(v, 'render'):
                rval[k] = yield v.render()
            elif not isinstance(v, str):
                rval[k] = v
            elif hasattr(self, v):
                if callable(getattr(self, v)):
                    rval[k] = yield getattr(self, v)()
                else:
                    rval[k] = yield getattr(self, v)
            else:
                rval[k] = v
        return rval

    def __getattr__(self, item):
        if item in self.items.keys():
            v = self.items[item]
            if callable(v):
                return v()
            if not isinstance(v, str):
                return v
            if callable(getattr(self, v)):
                rval = getattr(self, v)()
            else:
                rval = getattr(self, v)
            return rval
        raise AttributeError





from .base import SysInfoBase
from tendril.utils import versions


class VersionsInfo(SysInfoBase):
    def __init__(self, *args):
        super(VersionsInfo, self).__init__(*args)
        self._namespaces = []
        self._packages = []

    def register_namespace(self, namespace):
        if namespace not in self._namespaces:
            self._namespaces.append(namespace)

    def register_package(self, package):
        if package not in self._packages:
            self._packages.append(package)

    @property
    def items(self):
        if not self._items:
            for package in self._packages:
                self._items[package] = versions.get_version(package)
            for namespace in self._namespaces:
                self._items[namespace] = {k: v for k, v in versions.get_versions(namespace)}
        return self._items





from .base import SysInfoBase


class ConfigInfo(SysInfoBase):
    def __init__(self, *args):
        super(ConfigInfo, self).__init__(*args)

    @property
    def items(self):
        rv = {}
        for key in self.actual.config._elements.keys():
            rv[key] = getattr(self.actual.config, key)
        return rv





import pyedid
from functools import partial
from glob import glob
from ebs.linuxnode.core.config import ElementSpec, ItemSpec

from .base import SysInfoBase


class DisplayInfo(SysInfoBase):
    def __init__(self, *args):
        super(DisplayInfo, self).__init__(*args)
        self._names = []

    def install(self):
        super(DisplayInfo, self).install()

        _elements = {
            'sysinfo_display': ElementSpec('sysinfo', 'display', ItemSpec(fallback=None)),
        }

        for element, element_spec in _elements.items():
            self.actual.config.register_element(element, element_spec)

        if self.actual.config.sysinfo_display:
            candidates = [self.actual.config.sysinfo_display]
        else:
            candidates = glob('/sys/class/drm/*-HDMI-*/edid') + \
                glob('/sys/class/drm/*-DP-*/edid')
        for candidate in candidates:
            name = candidate[len('/sys/class/drm/'):(-1 * len('/edid'))]
            self._items[name] = partial(self._display_edid, candidate)
            self._names.append(name)

        self._items['connected'] = self._display_connected

    def _display_edid(self, path):
        with open(path, 'rb') as f:
            content = f.read()
        try:
            edid_info = pyedid.parse_edid(content)
            rv = {
                'detected': True, 'serial': edid_info.serial,
                'product_id': edid_info.product_id, 'manufacturer_id': edid_info.manufacturer_id,
                'name': edid_info.name, 'manufacturer': edid_info.manufacturer,
                'width': edid_info.width, 'height': edid_info.height,
            }
            return rv
        except ValueError:
            return {'detected': False}

    def _display_connected(self):
        for name in self._names:
            info = getattr(self, name)
            if info['detected']:
                return True
        else:
            return False





import psutil
from psutil._common import bytes2human
from .base import SysInfoBase


class MemoryInfo(SysInfoBase):
    def __init__(self, *args):
        super(MemoryInfo, self).__init__(*args)

    def install(self):
        super(MemoryInfo, self).install()
        self._items = {
            'capacity': self._capacity,
            'available': self._available
        }

    def _capacity(self):
        return bytes2human(psutil.virtual_memory().total)

    def _available(self):
        return bytes2human(psutil.virtual_memory().available)





import psutil
import time
from .temperature import TemperatureInfo
from .disks import DiskInfo
from .memory import MemoryInfo
from .display import DisplayInfo
from .base import SysInfoBase


class StatusInfo(SysInfoBase):
    def __init__(self, *args):
        super(StatusInfo, self).__init__(*args)

    def install(self):
        super(StatusInfo, self).install()
        temperature = TemperatureInfo(self.actual)
        temperature.install()
        disks = DiskInfo(self.actual)
        disks.install()
        memory = MemoryInfo(self.actual)
        memory.install()
        display = DisplayInfo(self.actual)
        display.install()
        self._items = {
            'uptime': self._uptime,
            'disks': disks,
            'memory': memory,
            'display': display,
            'temperature': temperature,
        }

    def _uptime(self):
        return time.time() - psutil.boot_time()





import ifcfg
from twisted.internet.defer import inlineCallbacks
from ebs.linuxnode.core.config import ElementSpec, ItemSpec
from .base import SysInfoBase


class WifiNetworkInfo(SysInfoBase):
    @property
    def wifi_ssid(self):
        def _handle_result(result):
            return result.decode().strip()
        d = self._shell_execute(['iwgetid', '-s'], _handle_result)
        return d


class NetworkInfo(WifiNetworkInfo):
    def install(self):
        super(NetworkInfo, self).install()
        _elements = {
            'network_interface_wifi': ElementSpec('network', 'wifi', ItemSpec(fallback='wlan0')),
            'network_interface_ethernet': ElementSpec('network', 'ethernet', ItemSpec(fallback='eth0')),
            'network_interfaces': ElementSpec('_derived', self._network_interfaces)
        }
        for name, spec in _elements.items():
            self.actual.config.register_element(name, spec)

        self._items = {
            'info': 'network_info',
            'interfaces': 'network_interfaces',
        }

    @staticmethod
    def _network_check_interface(interface):
        if_spec = ifcfg.interfaces().get(interface, None)
        if not if_spec:
            return
        if_flags = if_spec['flags'].split('<')[1].split('>')[0].split(',')
        if "UP" in if_flags and "RUNNING" in if_flags:
            return True
        else:
            return False

    @staticmethod
    def _network_get_ipaddress(interface):
        if_spec = ifcfg.interfaces().get(interface, None)
        if not if_spec:
            return
        return if_spec['inet']

    def _network_interfaces(self, config):
        return [config.network_interface_wifi, config.network_interface_ethernet]

    def network_interfaces(self):
        return self.actual.config.network_interfaces

    def network_interfaces_wifi(self):
        return [self.actual.config.network_interface_wifi]

    def network_interfaces_ethernet(self):
        return [self.actual.config.network_interface_ethernet]

    @inlineCallbacks
    def network_info(self):
        rv = {}
        for interface in self.network_interfaces():
            if self._network_check_interface(interface):
                rv['interface'] = interface
                rv['ipaddress'] = self._network_get_ipaddress(interface)
                if interface in self.network_interfaces_wifi():
                    rv['ssid'] = yield self.wifi_ssid
        return rv





import json
from twisted.internet import reactor
from twisted.internet.defer import inlineCallbacks

from ebs.linuxnode.core import config
from ebs.linuxnode.core.basenode import BaseIoTNode

from ebs.linuxnode.sysinfo import SysinfoMixin


class ExampleNode(SysinfoMixin, BaseIoTNode):
    @inlineCallbacks
    def test_sysinfo(self):
        print("Executing Sysinfo Modules")
        sysinfo = yield self.sysinfo.render()
        print(json.dumps(sysinfo, indent=4))
        print(self.network_info)

    def start(self):
        self.install()
        super(ExampleNode, self).start()
        self.config.print()
        reactor.callLater(5, self.test_sysinfo)
        reactor.callLater(60, self.stop)
        reactor.run()

    def stop(self):
        super(ExampleNode, self).stop()
        reactor.stop()


def main():
    nodeconfig = config.IoTNodeConfig()
    config.current_config = nodeconfig

    node = ExampleNode(reactor=reactor)
    node.start()


if __name__ == '__main__':
    main()



import setuptools

_requires = [
    'setuptools-scm',
    'ebs-linuxnode-core',
    'raspi_system>=0.1.2',  # Only used on RPi
    'ifcfg',
    'tendril-utils-core',
    'psutil',
    'pyedid',
]

setuptools.setup(
    name='ebs-linuxnode-sysinfo',
    url='https://github.com/ebs-universe/ebs-linuxnode-sysinfo',

    author='Chintalagiri Shashank',
    author_email='shashank.chintalagiri@gmail.com',

    description='System Information Providers for EBS IOT Applications',
    long_description='',

    packages=setuptools.find_packages(),
    install_requires=_requires,

    setup_requires=['setuptools_scm'],
    use_scm_version=True,

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Operating System :: POSIX :: Linux',
    ],
)





import json
from twisted import logger
from twisted.internet.defer import inlineCallbacks
from ebs.linuxnode.core.shell import BaseShellMixin

from .base import SysInfoBase

from .host import HostInfo
from .network import NetworkInfo
from .status import StatusInfo
from .app import AppInfo


class SysinfoContainer(SysInfoBase):
    def install_module(self, name, module):
        m = module(self)
        m.install()
        self._items[name] = m

    def install(self):
        self.install_module('app', AppInfo)
        self.install_module('host', HostInfo)
        self.install_module('network', NetworkInfo)
        self.install_module('status', StatusInfo)

    @property
    def log(self):
        if not self._log:
            self._log = logger.Logger(namespace="sysinfo", source=self)
        return self._log

    @inlineCallbacks
    def write_to_log(self):
        sysinfo = yield self.render()
        self.log.info("System Information : {sysinfo}",
                      sysinfo=json.dumps(sysinfo, indent=2))


class SysinfoMixin(BaseShellMixin):
    def __init__(self, *args, **kwargs):
        super(SysinfoMixin, self).__init__(*args, **kwargs)
        self._sysinfo = SysinfoContainer(self)

    @property
    def sysinfo(self):
        return self._sysinfo

    def install(self):
        super(SysinfoMixin, self).install()
        self.sysinfo.install()
        self.sysinfo.app.versions.register_namespace('ebs')
        self.sysinfo.app.versions.register_namespace('tendril')
        self.reactor.callWhenRunning(self.sysinfo.write_to_log)

    @property
    @inlineCallbacks
    def network_info(self):
        # Legacy support
        info = yield self.sysinfo.network.info
        if 'ssid' in info.keys():
            return info['ssid']
        else:
            return info['ipaddress']





import psutil
from psutil._common import bytes2human
from .base import SysInfoBase


class DiskInfo(SysInfoBase):
    def __init__(self, *args):
        super(DiskInfo, self).__init__(*args)

    def install(self):
        super(DiskInfo, self).install()
        self._items = {
            'capacity': self._capacity,
            'free': self._free
        }

    def _capacity(self):
        return bytes2human(psutil.disk_usage('/').total)

    def _free(self):
        return bytes2human(psutil.disk_usage('/').free)
